
#ifndef _KISS_CLASSES_H_
#define _KISS_CLASSES_H_

/*

	Main KISS layout header:
		KissWindow, KissBox,       [necessary]
		KissMenuBar, KissStatusBox [accessory]
	Version 3.0+
	
	© 1999-2000 Cédric Degéa. All Rights Reserved.
	
	LICENSE: get in touch with me at degea@kagi.com for any use
		of this files or of any KISS headers, or the associated shared library.

*/

#include <interface/Box.h>
#include <interface/Window.h>
#include <interface/TextControl.h>

// base class
#include <Application.h>




class KISSApplication : public BApplication 
{
 public:
	KISSApplication();
	
	static void _AboutRequester();
};


	class KissVerboseTextControl : public BTextControl
	{
	public:
		KissVerboseTextControl( BRect frame,
							const char *name,
							const char *label, 
							const char *initial_text, 
							BMessenger mger,
							/*uint32*/BMessage replymsg )
		: BTextControl( frame, name, label, initial_text, new BMessage(replymsg) )
		{
			SetTarget(mger);
		} 
		
		virtual	status_t Invoke( BMessage *msg0 = NULL)
		{
			// Quick'n'dirty.. hacks to fix
			
			int8 byte=0;
			Window()->CurrentMessage()->FindInt8( "byte", &byte );
			if( byte != '\n' )
				return BTextControl::Invoke( msg0 );
			
			// do things in the right order
			Window()->PostMessage(B_QUIT_REQUESTED);
			
			BMessage *msg = new BMessage( *Message() );
			msg->AddString("logtext", Text());
			SetMessage(msg);
				//Messenger().SendMessage( msg );
			return BTextControl::Invoke( msg0 );
		}
	};



struct KissTok // a namespace for the Kiss Toolkit.
{
	static void OkAlert( const char * string, const char * str_if_is_error = NULL );
	static bool ConfirmAlert( const char *msg );
//	static void AskString( BMessenger mger, uint32 replymsg, const char * initialstr=NULL );
	static void AskString( BMessenger mger, BMessage replymsg, const char * initialstr=NULL );
};



class KissWindow : public BWindow
{
public:
	KissWindow(
		const char *title,
		bool 		make_a_statusbar = false,
		const char * settings_name = NULL,
		int32		id=0 );
	
	~KissWindow();
	
	
// new api
// .box
	void SetRoot( class KissBox * root );
	
// .menus
	BMenu * MenuBegin( const char * label, int32 msgcmd = 0, char shortcut = 0 /* less frequently used than by items.. */ );
	void    MenuEnd();
	BMenuItem * MenuAddEntry( const char * label, int32 msgcmd = 0, char shortcut = 0 );
	void MenuAddSeparator();

/*
// OLD
	void Build(
		const Brick *tags_array,
		void *the_global_struct,
		void *your_private_struct,
		int32 sizeof_struct) ;
*/
	
	void SetStatus(const char *statustext) ;
	
	void InvalidateLayout() ;
	
	// Class vars and methods : about unique-instance windows.
	static BWindow * ShowUnique(int32 id, bool activate);

private:

	KissBox * theRootBox ;

	class KissStatusBox	*statusbar ;
// modified policy : NULL if no menu in the window
	class BMenuBar * menuBar;
	float ascent, linehei ;
	
	const char * const settingsPath ;
	const int32 uniqueID ;
	
	// A KissWindow list (only the ones with registered IDs).
	// Appended by ctor(), reduced by dtor(), checked by ShowUnique().
	static class BList registeredWins ;

// new api for menus
	BMenu * stateMenu;
} ;



#endif // pragma once
