
#ifndef KISS_BOX_H_
#define KISS_BOX_H_


// base class
#include <app/MessageFilter.h>

// members
#include <support/String.h>

// ?
#include <interface/ScrollView.h>

//
#include "KISSlib.h"




class HelptextFilter : public BMessageFilter
{
	// Used by KissBox when ading children to itself so that they manage
	// their HelpText, w/o having to subclass them.
	
public:
	// Xtors
	HelptextFilter( const char * helptext /* will be copied */ );
	
	// Be overrides
	filter_result Filter( BMessage * msg, BHandler ** target );

private:
// const, or setter exists ?
	BString helpText;
};



/////
//// CONTAINER : The class on which the whole layout relies,
///  nodes and leaves : Named and UnNamed Boxen
//

class KissBox : public BBox
{
	typedef BBox inherited;
	
 public:

	KissBox();
//	static KissBox * Instantiate( BMessage * m );


virtual	int32		Proportion() const ;
	void		SetProportion(int32) ;
	int32		SumOfChildProportions() const ;
	void		SetHelpText(const char * const text) ;
	//const char *HelpText() const ;
	bool IsVertical() const;
	
	// nothing to do with their BControl counterparts in BeOS..
	// (no pre-existing overriding, but re-implementation..)
		//tunred into a virtual in 3.1b7 for charts augment
	virtual	void		SetEnabled(bool) ;
	bool IsEnabled() const;
	
//  private ..etc
	void layoutChildren();
	void childrenChanged();
	
	void InvalidateLayout() ;
	void LayoutDimsChanged();
// protected ?
	void setLayoutDims( int32 wid, int32 hei );
	
	void SetHelpTextOf ( const char * helptext, BView * view );
	
	virtual void GetPreferredSize(float *wid, float *hei) ;
	virtual void FrameResized(float new_width, float new_height) ;

	
	status_t ArchiveState( BMessage * archive ) const ;
	status_t UnarchiveState( const BMessage * archive, int32 index = 0 );
	
	KissBox      * OpenVerticalBox  ( const char * label = NULL, int32 proportion = 0, const char *helptext = NULL );
	KissBox      * OpenHorizontalBox( const char * label = NULL, int32 proportion = 0, const char *helptext = NULL );
	void           CloseBox();

public:
	void           SetDefaultMsgCmd( int32 msgwhat );
	

	BStringView * AddStringView( const char * label = NULL, const char * helptext = NULL );
	BListView * AddListView( const char * name = NULL, const char * helptext = NULL );
	BOutlineListView * AddOutlineListView( const char * name = NULL, const char * helptext = NULL );
	
	BView * AddView( BView * view, BView * before = NULL, const char * helptext = NULL );
	bool RemoveView( BView * view );
	
	BButton * AddButton( const char * label, const char * name = NULL, const char * helptext = NULL, int32 msgcmd = 0 );
	// Archivables :
	BCheckBox * AddCheckBox( const char * label, const char * name = NULL, const char * helptext = NULL, int32 msgcmd = 0 );
	BRadioButton * AddRadioButton( const char * label, const char * name = NULL, const char * helptext = NULL, int32 msgcmd = 0 );
	BTextControl * AddTextControl( const char * label, const char * name = NULL, const char * helptext = NULL, int32 msgcmd = 0 );
	BOptionPopUp * AddOptionPup( const char * label, const char * name = NULL, const char * helptext = NULL, int32 msgcmd = 0 );
	BPopUpMenu * AddPopUp( const char * label, const char * name = NULL, const char * helptext = NULL );
	BColorControl * AddColorControl( /*no label, */ const char * name = NULL, const char * helptext = NULL, int32 msgcmd = 0 );
	class KissDatePups * AddDatePups( const char * label, const char * name = NULL, const char * helptext = NULL, int32 msgcmd = 0 );
	
	// new in v3.1b
	class KissTabView * ConvertToTabview();
	
	static void _SetLicenseKey( int32 key );
	
protected:
	KissBox * OpenBox( const char * name = NULL, int32 proportion = 0, const char *helptext = NULL );
	bool isVertical ;
	
private:
	BMessage * messageWithDefaultAnd( int32 );

	float labelHeight() const ;
	float normalGap() const ;
	
	//
	bool isEnabled;
	
	KissBox * state_box ;
	int32 msgWhat; // default 'what' of BMsgs of created widgets by this Kissbox

	float linehei ;
	float ascent ;
	float minimum_w, minimum_h ;
	int32 proportion ;
	//const char *helpText ;
};



// Export as may be useful to others (for instance, FAMe/LVRecord)
class KissBoxFixed : public KissBox
{
	// A special KissBox (especially for CScrollView to always return a _1_ proportion..
	// All this to save a dynamic_cast in the live-layout code :-)
public:
	int32		Proportion() const { return 1;};
};


class CScrollView : public BScrollView, public KissBoxFixed
{
	// A BScrollView with a correct behaviour in GetPreferredSize() (sigh..)
public:
	CScrollView(const char *name,
								BView *target,
								uint32 resizeMask = B_FOLLOW_LEFT |
													B_FOLLOW_TOP,
								uint32 flags = 0,
								bool horizontal = false,
								bool vertical = false, 
								border_style border = B_FANCY_BORDER)
	:	BScrollView(name, target, resizeMask, flags, horizontal, vertical, border)
	{
		// do nothing more
	}
	virtual void GetPreferredSize(float *wid, float *hei)
	{
		// inherited:
		// nope! pure override..
		
		*wid = *hei = 50;
	}
};



#endif // pragma once
