
// libnetapi.so
#include <net/NetAddress.h>
#include <net/NetEndpoint.h>
// misc
#include <assert.h>
#include <stdio.h>
#  include <stdlib.h>
#include <storage/FindDirectory.h> // to find the settings dir.
#include <storage/Directory.h>  // create_directory()
#include <storage/Path.h>
#include <support/String.h>
// project
#include "MainWindow.h"
#include "PrefsWindow.h"


//#error define your own vals here:
//#define _USER "??"
//#define _PASS "??"
#define _SERVER "news.free.fr"

#define APP_SUBDIR "Woogie"



const char * func( const char * login, const char * password );


/********************/

class App : public BApplication
{
public:
	// Xtors
	App();
	
	// Be overrides
	void ReadyToRun();

private:
};

App::App()
:	BApplication( "application/x.vnd-user.woogie" )
{
	MainWindow * win = new MainWindow();
	win->Show();
	
	PrefsWindow *prefs = new PrefsWindow();
	prefs->Show();
	
	//
	// Prefs bookkeeping
	//
	
	BPath path ;
	
	// Whenever launched, ensure that our settings_folder exists.
	find_directory( B_USER_SETTINGS_DIRECTORY, &path );
	path.Append( APP_SUBDIR );
	create_directory( path.Path(), 0777 ); // ignore errors.
}

void App::ReadyToRun()
{
}


int main()
{
	new App();
	
	be_app->Run();
	
	return 0;
}

/*
int main()
{
	const char *err = func( _USER, _PASS );
	if( err )
		perror( err );
}
*/






class ServerDialog
{
public:
	ServerDialog( const char * ipname, int32 port );
	~ServerDialog();
	
	int32 Ask( const char * sendtext );
	const char * InitCheck();

private:
	BNetEndpoint server;
	BNetAddress addy;
	
	const char * initCheck;
};


ServerDialog::ServerDialog( const char * ipname, int32 port )
:	server(),
	addy( ipname, port ),
	initCheck( NULL )  // by default: success
{
	if( B_OK != server.InitCheck() ) {
		initCheck = "ctor";
		return;
	}
	if( B_OK != addy.InitCheck() ) {
		initCheck = "addy";
		return;
	}
	if( B_OK != server.Connect(addy) ) {
		initCheck = "connect";
		return;
	}
}


ServerDialog::~ServerDialog()
{
	server.Close();
}


const char * ServerDialog::InitCheck()
{
	return initCheck;
}


int32 ServerDialog::Ask( const char * sendtext )
{
	// Send question
	if( sendtext )
		server.Send( sendtext, strlen(sendtext) );
	
	// Receive answer
	BNetBuffer buffer;
	int32 rved = server.Receive( buffer, 1000000 ); // 1 MB upper limit
	
	//
	printf( "received %ld bytes\n", rved );
	printf(" buffer content: %s\n", buffer.Data() );
	//
	//server.SetTimeout(500000);
	//server.IsDataPending( 10000 ) )
	
	BString s_ret( (const char *)buffer.Data(), 3 );
	return atoi( s_ret.String() );
}




const char * func( const char * login, const char * password )
{
	assert(login);
	assert(password);
	
	ServerDialog serv( _SERVER, 119 );
	const char * err = serv.InitCheck();
	if( err )
		return err;
	
	int32 retcode=0;
	
	// ignore welcome message
	retcode = serv.Ask( NULL );
//retcode assert?
	
	// test it a bit
	retcode = serv.Ask( "HELP\r\n" );
	if( retcode == 480 )
	{
		BString user;
		user << "AUTHINFO USER " << login << "\r\n";
		retcode = serv.Ask( user.String() );
		if( retcode != 381 )
			return "failed after AUTHINFO USER";
		else
		{
			BString pass;
			pass << "AUTHINFO PASS " << password << "\r\n";
			retcode = serv.Ask( pass.String() );
			if( retcode != 281 )
				return "failed after AUTHINFO PASS";
			else
			{
				BString group;
//
				group << "GROUP fr.comp.sys.be \r\n";
				retcode = serv.Ask( group.String() );
				if( retcode != 9999 )
					return "what code to expect here??";
				
				
				// returns number of first and last post available:
				// buffer content: 211 1581 8324 9913 fr.comp.sys.be
				 
				/*
					STAT number_of_article  to set the "internal pointer" onto it
					NEXT => increases the "server internal pointer"
					#HEADER
					#BODY
					ARTICLE => get article at current pointer
					QUIT
				*/
			}
		}
	}
	//	else if..
	else
	{
		BString s_ret;
		s_ret << retcode;
//		KissTok::OkAlert( "bad return code from server! expected 480 or.., got", s_ret.String() );
		return "failed before authentication";
	}
	
	// finished, success
	return NULL;
}
